from dataEngine import Common, Event


ConversationsList = {}

class Conversation(Common.PbxDataObject):
    def __init__(self, ID, caller, destination):
        # print("====Init conversation " + str(ID))
        self.ID = ID
        self.status = "connecting"
        self.caller = caller
        self.callee = None
        self.app = None
        self.destination = destination
        self.service = None
        self.trunk = None
        self.queue = None
        self.tags = None
        self.flags = []
        self.splitReason = None
        self.transferType = None
        self.mergeWith = None
        self.cause = None
        self.causeStr = None
        self.who = None
        self.recording = None
        self.lookupInfo = None
        self.event = Event.Event(ID, self.status, caller.get_info(), destination)

    def set(self, param, value):
        #print("\n\n>>> Conversation.set, param: {}, value: {}".format(param,value))
        if not value or value == self.get(param) or (param == "destination" and value == "s"):
            return
        #print("\n\n>>> conversation, set: {}\n\n".format(self))
        super().set(param, value)
        self.event.set(param, value)
        if param == "status":
            self.event.push("CALL.UPDATE")

    def link_channel(self, participant, chan):
        #print(">>> link_channel participant: {}, chan: {}".format(participant, chan))
        if participant not in ("caller", "callee"):
            return
        self.set(participant, chan)
        self.set_participant_info(participant, chan.get_info())
        # merge callee info looked up before callee channel has started
        if self.lookupInfo is not None and participant == "callee":
            self.set_participant_info("callee", self.lookupInfo)
            self.lookupInfo = None

    def set_participant_info(self, participant, info):
        #print(">>> set_participant_info participant: {}, info: {}".format(participant,info))
        if participant not in ("caller", "callee"):
            return
        if self.get(participant) is None:
            if participant == "callee":
                # store callee info, it will be processed upon callee channel start
                self.lookupInfo = info
        else:            
            for key, value in info.items():
                if value:
                    self.get(participant).set(key, value)
                    self.event.get(participant)[key] = value

    def set_flags(self, value):
        self.flags.append(value)
        self.event.flags.append(value)

    def set_app(self, app, destination):
        if app not in ("Dial", "Queue", "Page", "ConfBridge", "VoiceMail"):
            return
        self.app = app
        self.event.app = app
        if self.app is None:
            if self.destination != destination:
                self.set("destination", destination)
                #print("\n\n>>> conversation: {}\n\n".format(self))
                self.event.push("CALL.UPDATE")
        elif app == "VoiceMail":
            self.set_flags("voicemail")
            self.set("destination", destination)
        else:
            if self.destination != destination:
                self.set("destination", destination)
                self.conv_split("dialplan")

    def conv_split(self, reason):
        self.status = "connecting"
        self.event.status = "connecting"
        self.splitReason = reason
        self.event.reason = reason
        #print("\n\n>>> conversation: {}\n\n".format(self))
        self.event.push("CALL.SPLIT")

    def record_start(self):
        self.recording = "started"
        self.event.push("CALL.RECORD_START")

    def record_stop(self, recording):
        self.recording = recording
        self.event.recording = recording
        self.event.push("CALL.RECORD_STOP")
        if self.status == "end":
            ConversationsList.pop(self.ID, None)

def add(conv):
    ConversationsList[conv.ID] = conv
    #print("\n\n>>> conversation: {}\n\n".format(conv))
    conv.event.push("CALL.START")

def get(ID):
    return ConversationsList.get(ID)

def end(ID):
    conv = ConversationsList.get(ID)
    if conv is None:
        return
    #print("\n\n>>> conversation: {}\n\n".format(conv))
    conv.event.push("CALL.END")
    conv.status = "end"
    if conv.recording != "started" and ID in ConversationsList:
        del ConversationsList[ID]

def print_currect_state():
    res = "ConversationsList:"
    for c in ConversationsList.keys():
        res += "\n\n{}".format(ConversationsList[c])
    print("===={}".format(res))
