from datetime import datetime
import json
from dataEngine import Common, Aws, Channel, AppConfig


class Event(Common.PbxDataObject):
    def  __init__(self, ID, status, caller, destination):
        # print("====Init event " + str(ID))
        self.ID = ID
        self.name = None
        self.status = status
        self.service = None
        self.app = None
        self.destination = destination
        self.trunk = None
        self.queue = None
        self.tags = None
        self.season = None
        self.transferType = None
        self.recording = None
        self.mergeWith = None
        self.cause = None
        self.causeStr = None
        self.who = None
        self._caller = caller
        self._callee = {}
        self._flags = []

    @property
    def caller(self):
        return self._caller

    @caller.setter
    def caller(self, new_val):
        if new_val is None:
            self._caller = {}
        else:
            self._caller = new_val

    @property
    def callee(self):
        return self._callee

    @callee.setter
    def callee(self, new_val):
        if new_val is None:
            self._callee = {}
        else:
            self._callee = new_val

    @property
    def flags(self):
        return self._flags

    @flags.setter
    def flags(self, new_val):
        if new_val is None:
            self._flags = []
        else:
            self._flags = new_val

    def _get_data(self, attrs):
        rv = {}
        for attr in attrs:
            value = self.get(attr)
            if value:
                rv[attr] = value
                setattr(self, attr, None)
        return rv

    # START/UPDATE
    def get_data(self):
        attrs = ["status","caller","callee","service","destination","trunk","queue","tags","flags"]
        return self._get_data(attrs)

    def get_data_split(self):
        attrs = ["reason","service","transferType","mergeWith","status","caller","callee","destination","trunk","queue","tags","flags"]
        return self._get_data(attrs)

    def get_data_agent(self):
        return {}

    def get_data_record_stop(self):
        rv = {}
        if self.recording is not None:
            rv["fileName"] = value
            self.fileName = None
        return rv

    def get_data_end(self):
        attrs = ["cause","causeStr","who"]
        return self._get_data(attrs)

    def push(self, name):
        #print("\n\n>>> Event.push, self: {}\n\n".format(self))
        data = None
        if name in ("CALL.START", "CALL.UPDATE"):
            data = self.get_data()
        elif name == "CALL.SPLIT":
            data = self.get_data_split()
        elif name == "CALL.RECORD_STOP":
            data = self.get_data_record_stop()
        elif name == "CALL.END":
            data = self.get_data_end()

        push(name, self.ID, data)

def push(name, id, data):
    ev = {
        "ext" if name == "PRESENCE" else "id": id,
        "type": name,
        "time": int(datetime.timestamp(datetime.now())*1000),
        "pbx": Common.PBX_SERIAL
    }
    if data:
        ev["data"] = data

    payload = json.dumps(ev, cls=Common.PbxDataEncoder)
    if AppConfig.get("out_file"):
        with open(AppConfig.get("out_file"), "a") as log_file:
            log_file.write("{}\n".format(payload))
    print("\n\033[92m{},\033[0m".format(payload))
    if not AppConfig.isPushDisabled():
        payload_bytes = payload.encode('utf-8')
        put_response = Aws.KINESIS_CLIENT.put_record(
            StreamName=Aws.AWS_STREAM,
            Data=payload_bytes,
            PartitionKey=Common.PBX_SERIAL)
    #print("Response: " + str(put_response))
