import subprocess, re
from ldap3 import Server, Connection, SAFE_SYNC
from dataEngine import AppConfig

# LDAP
LDAP_USER = subprocess.getoutput("cat /etc/callweaver/res_ldap.conf | grep 'user=' | cut -d= -f 2,3,4")
LDAP_PASS = subprocess.getoutput("cat /etc/callweaver/res_ldap.conf | grep 'pass=' | cut -d= -f 2")
LDAP_SERVER = Server("localhost")
LDAP_CONN = None

ATTR_MAP = {
    "uid": "userID",
    "mail": "email",
    "telephoneNumber": "userExtension",
    "userKind": "userKind",
    "cn": "commonName",
    "mobile": "mobile",
    "ou": "department"
}

USER_ATTRIBUTES = list(ATTR_MAP.keys())

def _get_connection():
    global LDAP_CONN
    if LDAP_CONN is None:
        LDAP_CONN = Connection(LDAP_SERVER, LDAP_USER, LDAP_PASS, client_strategy=SAFE_SYNC, auto_bind=True)
    return LDAP_CONN

def getUserInfo(number):
    if AppConfig.isTest():
        return {"userID": number, "email": "{}@wildix.com".format(number), "userExtension": number, "userKind": "user",
                "commonName": "User {}".format(number),"groupID": "000000{}".format(number), "groupName": "Default",
                "department": "dummyDepartment"}
    status, _, response, _ = _get_connection().search(search_base="dc=wildix",
                                              search_filter="(&(objectClass=wPerson)(|(telephoneNumber={})(mobile={})))".format(
                                                  number, number), attributes=USER_ATTRIBUTES+["owner"])
    if status and response:
        rv = {ATTR_MAP[attr]:(response[0].get("attributes").get(attr)[0]) for attr in USER_ATTRIBUTES if response[0].get("attributes").get(attr)}
        if "userID" in rv:
            gid = re.findall("^gid=(\d+),.*", response[0].get("attributes").get("owner")[0])
            if gid:
                rv["groupID"] = gid[0]
                rv["groupName"] = _getUserGroupName(gid[0])
        return rv
    else:
        return {}

def _getUserGroupName(gid):
    if AppConfig.isTest():
        return "Default"
    status, _, response, _ = _get_connection().search(search_base="dc=wildix",
                                              search_filter="(&(objectClass=wGroup)(gid={}))".format(gid),
                                              attributes=["cn"])
    if status and response and "cn" in response[0].get("attributes"):
        return response[0].get("attributes").get("cn")[0]
    return
