#!/usr/bin/python3
import sys, time, syslog, base64, os
import argparse, json, random, traceback, re
from dataEngine import Common, Aws, Ami, Ldap, Conversation, Channel, Socket, Event, AppConfig, AmiMock, PcapReader

re._pattern_type = re.Pattern

# TODO
# 1. Use environment variable for aws_access_key_id and aws_secret_access_key
# 2. add queue to push messages to kinesis, send it by batch to reduce payments
# 3. Add systemd service for use this script
# 4. Add all python's dependencies (boto3, asterisk.ami, ldap3) to deb package
# 5. Add cache for LDAP queries
# 6. Add BRIDGE_START logic for app queue (Local channels)

# CHECK
# 1. All kind of transfers
#   1.1. Hardware transfer (blind / attendant / semy attendant transfer) -- DONE
#   1.2. Collaboraation transfer (blind / attendant / semy attendant transfer) -- DONE
#   1.3. Feature сode (blind / attendant / semy attendant transfer) -- DONE
# 2. REFER after DirectRTP -- DONE
# 3. Pickup -- DONE
# 4. Forward -- DONE
# 5. Redirect -- DONE
#####

def event_listener(event, **kwargs):
    amiEvent = event
    #print("\n\n====----------- Got amiEvent: {}\n".format(amiEvent))
    try:
        # PresenceEvent
        if amiEvent.name == "PresenceEvent":
            handler_presence(amiEvent)
            return

        LinkedID = get_param(amiEvent, "LinkedID")
        convObj = Conversation.get(LinkedID)
        if convObj is None and get_param(amiEvent,"EventName") != "CHAN_START":
            return

        if amiEvent.name == "CEL":
            handler_cel(amiEvent)
        # BRIDGE_START
        elif amiEvent.name == "BRIDGE_START":
            #print("\n\n>>> {}\n\n".format(amiEvent))
            UniqueID1 = get_param(amiEvent, "UniqueID1")
            UniqueID2 = get_param(amiEvent, "UniqueID2")
            chanObj1 = Channel.get(UniqueID1)
            chanObj2 = Channel.get(UniqueID2)
            if chanObj2.get("chanType") != Channel.ChanType.LOCAL:
                if UniqueID1 != LinkedID: #TODO need to recheck logic "who is caller?"
                    convObj.link_channel("caller",chanObj1)
                convObj.link_channel("callee",chanObj2)
                convObj.set("status","talking")
        # BRIDGE_UPDATE
        elif amiEvent.name == "BRIDGE_UPDATE":
            UniqueID1 = get_param(amiEvent, "UniqueID1")
            UniqueID2 = get_param(amiEvent, "UniqueID2")
            callerObj = Channel.get(UniqueID1)
            if UniqueID2 != LinkedID and callerObj.get("chanType") != Channel.ChanType.LOCAL:
                convObj.link_channel("caller",callerObj)
                calleeObj = Channel.get(UniqueID2)            
                convObj.link_channel("callee",calleeObj)
                convObj.set("status","talking")
        # HOLD/UNHOLD
        elif amiEvent.name == "MusicOnHold":
            state = get_param(amiEvent, "State")
            if state == "Start":
                convObj.set("status","hold")
            elif state == "Stop":
                convObj.set("status","talking")
        # RECORD_START
        elif amiEvent.name == "MonitorStart":
            Conversation.get(LinkedID).record_start()
        # RECORD_STOP
        elif amiEvent.name == "MonitorStop":
            recording = get_param(amiEvent, "FileName")
            Conversation.get(LinkedID).record_stop(recording)
        # UniqueIDMasquerade
        elif amiEvent.name == "UniqueIDMasquerade":
            UniqueID1 = get_param(amiEvent, "UniqueID1")
            UniqueID2 = get_param(amiEvent, "UniqueID2")
            Channel.uid_masquerade(UniqueID1, UniqueID2, LinkedID)
        """if amiEvent.name in {"BRIDGE_START","BRIDGE_UPDATE","UniqueIDMasquerade"}:
            print("\n\n>>> handler_cel, \n convObj: \n{}\n".format(convObj))
            print("\n>>> event_listener, \n caller: {}\n callee: {}\n".format(convObj.get("caller"),convObj.get("callee")))"""

    except AttributeError as e1:
        print("ERROR: {}".format(e1))
        Channel.print_currect_state()
        Conversation.print_currect_state()
        syslog.syslog(syslog.LOG_ERR, 'AttributeError: {}'.format(e1))
    except Exception as e:
        print("ERROR: ", e, str(traceback.format_exc()))
        syslog.syslog(syslog.LOG_ERR, 'Exception: {}'.format(e))

def handler_cel(amiEvent):
    amiEventName = get_param(amiEvent, "EventName")
    UniqueID = get_param(amiEvent, "UniqueID")
    LinkedID = get_param(amiEvent, "LinkedID")
    convObj = Conversation.get(LinkedID)
    # CHAN_START
    if amiEventName == "CHAN_START":
        CallerIDnum = get_param(amiEvent, "CallerIDnum")
        CallerIDname = get_param(amiEvent, "CallerIDname")
        destination = get_param(amiEvent, "Exten")
        context = get_param(amiEvent, "Context")
        userInfo = None
        if "mobility" == context:
            userInfo = Ldap.getUserInfo(destination)
            destination = "mobility"
            if CallerIDnum == "":
                CallerIDnum = userInfo.get("mobile")
            if CallerIDname == "":
                CallerIDname = userInfo.get("commonName")
        else:
            userInfo = Ldap.getUserInfo(CallerIDnum)
        chanName = get_param(amiEvent, "Channel")
        chanObj = Channel.Channel(
            uid=UniqueID,
            lid=LinkedID,
            userID=userInfo.get("userID"),
            email=userInfo.get("email"),
            userExtension=userInfo.get("userExtension"),
            groupID=userInfo.get("groupID"),
            groupName=userInfo.get("groupName"),
            phone=CallerIDnum,
            name=CallerIDname,
            chanType=Channel.ChanType.SIP if chanName.startswith("SIP/") else Channel.ChanType.LOCAL,
            chanName=chanName,
            context=context
        )
        Channel.add(chanObj)
        if not convObj:
            convObj = Conversation.Conversation(LinkedID, chanObj, destination)
            if context == "mobility":
                convObj.set_flags("mobility")
            Conversation.add(convObj)
    # TRUNK_CALL
    elif amiEventName == "TrunkCall":
        trunk = get_param(amiEvent, "Extra").split(",")
        try:
            convObj.set("trunk",{"direction":trunk[0],"name":trunk[1]})
        except IndexError:
            pass
    # LOOKUP
    elif amiEventName == "LookUp":
        extra = get_param(amiEvent, "Extra").split(",")
        lookupInfo = {
                "name":extra[0],
                "company":extra[1],
                "phone":extra[2],
                "email":extra[3]
            }
        if extra[4] == "inc":
            convObj.set_participant_info("caller",lookupInfo)
        elif extra[4] == "out":
            convObj.set_participant_info("callee",lookupInfo)
    # ADD_TAG
    elif amiEventName == "Tag":
        extra = get_param(amiEvent, "Extra")
        tags = extra.replace(" ", "").split(",")
        convObj.set("tags",tags)
    # App (UPDATE/SPLIT)
    elif amiEventName == "App":
        chanName = get_param(amiEvent, "Channel")
        if chanName.startswith("SIP/"):
            extra = get_param(amiEvent, "Extra").split(",")
            appName = extra[0]
            destination = re.findall(".*/(\+?\d+)", extra[1])
            if destination:
                convObj.set_app(appName, destination[0])
    # ANSWER
    elif amiEventName == "ANSWER":
        chanObj = Channel.get(UniqueID)
        application = get_param(amiEvent, "Application")
        if application == "VoiceMail":
            convObj.set("status","talking")
    # CALL_GROUP
    elif amiEventName == "CallGroup":
        queue = get_param(amiEvent, "Extra").split(",")
        try:
            convObj.set("queue",{"name":queue[0],"id":queue[1]})
        except IndexError:
            pass
    # FORWARD
    elif amiEventName == "FORWARD":
        convObj.set("destination",get_param(amiEvent, "Extra"))
        convObj.conv_split("forward")
    # TRANSFER | PARK_START
    elif amiEventName in ("BLINDTRANSFER", "ATTENDEDTRANSFER"):
        transferType = re.findall("^(\w+)TRANSFER", amiEventName)[0].lower()
        if transferType == "blind":
            extra = get_param(amiEvent, "Extra").split(",")
            destination = extra[0]
        elif transferType == "attended":
            destination = get_param(amiEvent, "Exten")     
        convObj.set("destination",destination)
        destInfo = Ldap.getUserInfo(destination)
        # PARK_START
        if "park_orbit" == destInfo.get("userKind"):
            convObj.conv_split("park")
        # BLIND TRANSFER
        elif transferType == "blind":
            convObj.set("transferType",transferType)
            convObj.set("causeStr","transfer")
            convObj.set("who","caller")
            # split current conversation in case blind transfer
            convObj.conv_split("transfer")
        # ATTENDED TRANSFER
        elif transferType == "attended":
            mergeWith = None
            if get_param(amiEvent, "Extra") != "":
                mergeWith = get_param(amiEvent, "Extra")
            elif get_param(amiEvent, "Peer") != "":
                mergeWith = Channel.get_by_name(get_param(amiEvent, "Peer")).get("lid")    
            # set mergeWith for current conversation
            convObj.set("mergeWith",mergeWith)
            convObj.set("transferType",transferType)
            convObj.set("causeStr","transfer")
            convObj.set("who","caller")
            convObj.conv_split("transfer")
            # for opposite conversation
            oppositeConvObj = Conversation.get(mergeWith)
            oppositeConvObj.set("mergeWith",LinkedID)
            oppositeConvObj.set("transferType",transferType)
            oppositeConvObj.set("causeStr","transfer")
            oppositeConvObj.set("who","caller")
            oppositeConvObj.conv_split("transfer")
    # PICKUP | PARK_END
    elif amiEventName == "PICKUP":
        destination = get_param(amiEvent, "Exten")
        destInfo = Ldap.getUserInfo(destination)
        convObj.set("destination",destination)
        # PARK END
        if "park_orbit" == destInfo.get("userKind"):
            convObj.conv_split("park_end")
        # PICKUP
        else:
            mergeWith = Channel.get_by_name(get_param(amiEvent, "Peer")).get("lid")
            convObj.set("mergeWith",mergeWith)
            targetConvObj = Conversation.get(mergeWith)
            # set hangup cause and hangup By for target conversation
            targetConvObj.set("causeStr","pickup")
            targetConvObj.set("who","caller")
            convObj.conv_split("pickup")
    # HANGUP
    elif amiEventName == "HANGUP":
        processHangup = None
        callerObj = convObj.get("caller")
        if callerObj is not None and callerObj.get("uid") == UniqueID:
            processHangup = "caller"
        calleeObj = convObj.get("callee")
        if calleeObj is not None and calleeObj.get("uid") == UniqueID:
            processHangup = "callee"
        if processHangup is not None:
            extra = get_param(amiEvent, "Extra")
            splitExtra = extra.split(",")
            causeCode = splitExtra[0]
            dialstatus = splitExtra[2]
            causeStr = "".join(splitExtra[3:])
            hangupBy = None
            if splitExtra[1] != "":
                if callerObj is not None and callerObj.get("chanName") == splitExtra[1]:
                    hangupBy = "caller"
                elif calleeObj is not None and calleeObj.get("chanName") == splitExtra[1]:
                    hangupBy = "callee"
            if hangupBy is None and convObj.get("mergeWith") is not None:
                if convObj.get("transferType") is not None:
                    hangupBy = "caller"
                    causeStr = "transfer"
            # set hangupBy for cases, when PBX ended the call
            if hangupBy is None:
                hangupBy = "callee"
            convObj.set("who",hangupBy)
            convObj.set("cause",causeCode)
            convObj.set("causeStr",causeStr)
    # CHAN_END
    elif amiEventName == "CHAN_END":
        Channel.end(UniqueID)
    # LINKEDID_END
    elif amiEventName == "LINKEDID_END":
        Conversation.end(LinkedID)
    # Debug
    """if amiEventName in {"CHAN_START","HANGUP","LINKEDID_END"}:
        print("\n\n>>> handler_cel, \n convObj: {}\n".format(convObj))
        print("\n>>> handler_cel, \n caller: {}\n callee: {}\n".format(convObj.get("caller"),convObj.get("callee")))"""

def handler_presence(amiEvent):
    def _serialize(o):
        res = {}
        if o.get("untilTime"):
            res["until"] = o.get("untilTime")
        if o.get("location"):
            tmp = json.JSONDecoder().decode(o.get("location"))
            if "" not in tmp:
                res["location"] = tmp
        if o.get("message"):
            res["message"] = o.get("message")
        return res

    data = {}
    try:
        message = base64.b64decode(get_param(amiEvent, "Message")).decode("utf-8")
        data = json.JSONDecoder(object_hook=_serialize).decode(message)
    except Exception:
        pass
    ext = get_param(amiEvent, "User")
    userInfo = Ldap.getUserInfo(ext)
    department = userInfo.get("department")
    if department:
        data["department"] = department
    data["group"] = userInfo.get("groupID")

    data["status"] = get_param(amiEvent, "State")
    Event.push("PRESENCE", get_param(amiEvent, "User"), data)

def get_param(amiEvent, name):
    val = amiEvent.keys.get(name)
    if not val:
        correction = {"UniqueID": "Uniqueid",
                      "LinkedID": "Linkedid"}
        val = amiEvent.keys.get(correction.get(name))
    if val:
        return val.replace("wildixbox-","")

###########

def stop(delay = None):
    msg = " with a delay of %d seconds" % delay if delay else ""
    print("Stop{}".format(msg))

    PcapReader.close()
    Ami.close_connection()
    Socket.close_connection()
    AmiMock.close_connection()

    if delay:
        time.sleep(delay)
    sys.exit()

def parse_args(argv):
    def str_to_bool(value):
        if isinstance(value, bool):
            return value
        if value.lower() in ('false', 'f', '0', 'no', 'n'):
            return False
        elif value.lower() in ('true', 't', '1', 'yes', 'y'):
            return True
        raise ValueError(f'{value} is not a valid boolean value')

    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--in_file', default=None,
                        help='pcap input file collected by `tcpdump -s0 -ilo port $(netstat -npa | grep 5039 | grep python | cut -d: -f2 | cut -d' ' -f1) -w IN_FILE`', )
    parser.add_argument('-o', '--out_file', help='path to output file with json events', default=None)
    parser.add_argument('-rwo', '--rewrite_out_file', nargs='?', type=str_to_bool, const=True, default=None,
                        help='rewrite output file')
    parser.add_argument('-dp', '--disable_push', help='disable push events to the Kinesis data stream',
                        type=str_to_bool, nargs='?', default=None, const=True)
    parser.add_argument('-p', '--port', help='port of pseudo AMI interface', type=int, default=None)
    args = parser.parse_args()
    print("args: {}".format(args))

    AppConfig.set("out_file", args.out_file)
    if args.in_file:
        if os.path.isfile(args.in_file):
            AppConfig.set("in_file", args.in_file)
            port = random.randint(7000, 65000)
            AppConfig.set("port", int(port))
            AppConfig.set("isTest", True)
            AppConfig.set("disable_push", True)
            AppConfig.set("rewrite_out_file", True)
        else:
            print("File '{}' does not exists".format(args.in_file))
            sys.exit(3)

    AppConfig.set("disable_push", args.disable_push)
    AppConfig.set("port", args.port)
    AppConfig.set("rewrite_out_file", args.rewrite_out_file)

def main(argv):
    print("Start")
    parse_args(argv)

    Socket.init_connection()
    if not Aws.check_cognito_credentials():
        stop(3600)
    Aws.init_client()

    if AppConfig.get("rewrite_out_file") and AppConfig.get("out_file") and os.path.isfile(AppConfig.get("out_file")):
        os.remove(AppConfig.get("out_file"))

    AmiMock.init_connection()
    Ami.init_connection(event_listener)
    PcapReader.init()

    try:
        while True:
            #Channel.print_currect_state()
            #Conversation.print_currect_state()
            if not Aws.check_cognito_credentials():
                stop(3600)
            if AppConfig.isStopped():
                break
            time.sleep(5)
    except (KeyboardInterrupt, SystemExit):
        pass

    stop()

if __name__ == "__main__":
    sys.exit(main(sys.argv))
